<?php
// To indicate we are accessing stuff via index.php.
define('INDEX_LOADED', true);

// Load (other) constants.
require './constants.php';

// Add class path to include path (order is important, make sure APP_PATH is in front so it is checked first).
set_include_path(APP_PATH.PS.get_include_path());

// Autoloader. Newer PHP versions (>= 5.3.0) support anonymous functions.
function my_autoloader($class) {
    spl_autoload(strtr($class, '_', DS));
}
spl_autoload_register('my_autoloader');

// Object creation and initialisation.
try {
    // Create config object.
    $config = new Config();
    // Load configuration.
    // Note that this require, together with the constants require above, are the only two requires that are needed;
    // the rest should be handled by the autoloader.
    require CFG_PATH;

    // Before we continue check if we are either online, or a developer.
    if ($config->online || in_array(Helper::getIp(), $config->developerIPs)) {
        // All is good.
    } else {
        // Include some static HTML file and stop execution.
        include './offline.htm';
        exit;
    }

    // Create database object. For now no fancy stuff, just PDO. You can create a wrapper if you want.
    $settings = $config->database;
    $database = new PDO(
        'mysql:'.
        'host='.$settings['host'].
        ';port='.$settings['port'].
        ';dbname='.$settings['database'].
        ';charset='.$settings['charset'],   // PHP >= 5.3.6
        $settings['user'],
        $settings['password'],
        $settings['options']
    );

    // Start session.
    session_start();

    // Load user object.
    $user = new User(array(
        'config'    => $config,
        'database'  => $database,
    ));

    // Load routing object.
    $routing = new Routing(array(
        'config'    => $config,
        'database'  => $database,
    ));

    // init user
    $user->init();

    // init routing
    $routing->init();

} catch (Exception $e) {
    // Something went wrong during the setup of the application.
    // The application might be in an unstable state, so we stop further execution.
    if (in_array(Helper::getIp(), $config->developerIPs)) {
        // @todo output escaping
        die('[error] '.$e->getMessage());
    } else {
        // Alternative: static page.
        die('[error] Application initialisation error.');
    }
}

// Run application.
try {
    // Routing (request - calculate requested page).
    $class = $routing->handleRequest();

    // Init page (response).
    ob_start($config->obOutputCallback);

    // Create page of requested type ($class).
    $page = new $class(array(
        'config'    => $config,
        'database'  => $database,
        'user'      => $user,
        'routing'   => $routing,
    ));

    // Execute page action.
    $page->exec();
} catch (Exception $e) {
    // Something went wrong during execution.
    try {
        // @todo: log error

        // Clean (and stop) all output buffering, then start it again.
        // We must start it again here, otherwise we might run into
        // content encoding errors (especially when you use ob_gzhandler).
        while (ob_get_level() > 0) {
            ob_end_clean();
        }
        ob_start($config->obOutputCallback);

        // If we are viewing the page from a developer IP, show error.
        if (in_array(Helper::getIp(), $config->developerIPs)) {
            // @todo output escaping
            die('[error] '.$e->getMessage());
        } else {
            // Otherwise attempt to show a neat error page.
            // Alternative: static page.
            $class = 'Page_Error_500';
            $page = new $class(array(
                'config'    => $config,
                'database'  => $database,
                'user'      => $user,
                'routing'   => $routing,
            ));
            // Execute page action.
            $page->exec();
        }
    } catch (Exception $e) {
        // Well, we tried (twice lol).
        // @todo: log error

        // Clean output again.
        while (ob_get_level() > 0) {
            ob_end_clean();
        }
        ob_start($config->obOutputCallback);

        if (in_array(Helper::getIp(), $config->developerIPs)) {
            // @todo output escaping
            die('[error] '.$e->getMessage());
        }
        die('[error] Non recoverable application runtime error. Sorry :(.');
    }
}