<?php
// When interacting with a user in code, one should always use $this->user, not $_SESSION['user'].
// $_SESSION['user'] is solely used to propagate the identity of a user.
class User
{
    protected $cfg;
    protected $db;
    protected $data; // user data, can be accessed directly by using $this->user->[fieldname]

    public function __construct($options) {
        $this->cfg  = $options['config'];
        $this->db   = $options['database'];
        $this->data = array();
    }

    public function init() {
        if (isset($_SESSION['user']['id']) && Helper::isIndex($_SESSION['user']['id'])) {
            // Check if we are dealing with an existing, valid user.
            /* @var $st PDOStatement */
            $st = $this->db->prepare(
                'SELECT usr_id, usr_name
                FROM users
                WHERE usr_id = ?'
            );
            $st->execute(array($_SESSION['user']['id']));
            if ($st->rowCount() == 1) {
                $row = $st->fetch();
                $this->data = array(
                    'id'    => $row['usr_id'],
                    'name'  => $row['usr_name'],
                );
            } else {
                $this->logout();
            }
        } else {
            // @todo logging or throw exception?
            $this->logout();
        }
    }

    // Tries to authenticate a user with the given credentials.
    // Returns the user id upon success or false.
    public function authenticate($username, $password) {
        $return = false;

        // Perform a CASE SENSITIVE user name check.
        // String comparisons on non-case-sensitive column types or collations are normally
        // case insensitive as well, but here we check things properly.
        /* @var $st PDOStatement */
        $st = $this->db->prepare(
            'SELECT usr_id, usr_password, usr_password_salt
            FROM users
            WHERE usr_name = ? COLLATE utf8_bin'
        );
        $st->execute(array(
            $username,
        ));
        if ($st->rowCount() == 1) {
            $row = $st->fetch();

            // Recreate password with submitted password and fetched salt.
            $data = self::createPassword($password, $row['usr_password_salt']);
            // Compare submitted password with actual password.
            if (strcmp($data['password'], $row['usr_password']) === 0) {
                // Passwords are equal so we return the user id.
                $return = $row['usr_id'];
            }
        }
        return $return;
    }

    // Performs all the tasks for logging a user in.
    public function login($userId) {
        // Note that when we log a user in this basically is a state change.
        // Therefore, the page should be refreshed in some way.
        // The init() routine will then kick in and afterwards you can use and up-to-date
        // user object via $this->user.
        session_regenerate_id(true);
        $_SESSION['user']['id'] = $userId;
    }

    public function logout() {
        // Note that when we log a user in this basically is a state change.
        // Therefore, the page should be refreshed in some way.
        // The init() routine will then kick in and afterwards you can use and up-to-date
        // user object via $this->user.
        unset($_SESSION['user']);
    }

    public function isLoggedIn() {
        return $this->__isset('id');
    }

    public function __set($key, $value) {
        $this->data[$key] = $value;
    }

    public function __get($key) {
        if ($this->__isset($key)) {
            return $this->data[$key];
        } else {
            throw new Exception('user key '.$key.' not set');
        }
    }

    public function __isset($key) {
        return array_key_exists($key, $this->data);
    }

    // Helper method.
    public static function isValidUsername($name) {
        /*
        - valid chars: 0-9a-z_-; case insensitive
        - minimal length: 3; max length: 30
        */
        return (preg_match('#^[a-z0-9_-]{3,30}$#i', $name) === 1);
    }

    // Helper method.
    public static function createPassword($password, $salt) {
        return array(
            'password'  => hash('sha256', $password.$salt),
            'salt'      => $salt,
        );
    }
}